/*
 This file is part of the digitalSTROM ds485-core library.

 Copyright (C) 2010 digitalSTROM.org, Schlieren, Switzerland

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation version 2
 of the License.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/time.h>

#include    <iomanip>
#include    <iostream>

using namespace std;


#define     DEF_READ_FD   0

int main(int argc, char** argv)
{
  int               escape = 0;
  int               sync   = 0;
  int               num;
  unsigned char     byte;
  int               fdRead = DEF_READ_FD;
  bool              timestamped = true;

  if( argc == 2 )
  {
    fdRead  = open( argv[1], O_RDONLY );
    if( fdRead < 0 )
    {
      perror( "open" );
      return -1;
    }
  }

  while( 1 )
  {
    num = read( fdRead, &byte, sizeof( byte ) );
    if( num < 0 )
    {
      perror( "read" );
      return    -1;
    }
    
    if( num == 0 )
    {
      usleep( 1000 );
      continue;
/*      printf( "No data read, exit\n" );
      return  -1;*/
    }

    if( byte == 0xfd )
    {
      sync    = 1;

      cout << endl;
      if( timestamped )
      {
        timeval start;
        gettimeofday(&start, 0);
        cout << dec << start.tv_sec << '.' << setw(3) << setfill('0') << dec << (start.tv_usec / 1000) << "; ";
      }
    }
    else if( byte == 0xfc )
    {
      escape  = 1;
    }
    else
    {
      if( !sync )
      { 
        continue;
      }
      if( escape )
      {
        byte = byte | 0x80;
        escape  = 0;
      }

      cout << setfill('0');
      cout << setw(2) << hex << (int)byte << " ";
    }
  }

  return    0;
}

